import os
from loguru import logger
from .log_entry import LogEntry

class FileLog78:
    def __init__(self, filename="7788_{time:YYYY-MM-DD_HH}.log", menu="logs"):
        self.menu = menu
        self.filename = filename
        self._configure_logger()

    def _configure_logger(self):
        log_directory = os.path.join(os.getcwd(), self.menu)
        os.makedirs(log_directory, exist_ok=True)
        log_file = os.path.join(log_directory, self.filename)
        
        logger.remove()  # 移除默认的处理器
        logger.add(
            log_file,
            rotation="1h",  # 每小时轮换一次
            retention="1 week",  # 保留一周的日志
            compression="zip",  # 压缩旧日志
            format="{message}",  # 只记录消息内容
            encoding="utf-8",  # 指定编码为 UTF-8
            serialize=False,  # 不序列化日志记录
            enqueue=True  # 使用队列来写入日志，避免 I/O 阻塞
        )

    def log_to_file(self, log_entry: LogEntry):
        logger.info(log_entry.to_json())

    def clear(self):
        # 使用 loguru 时，这个方法可能不需要实现
        pass

    def __del__(self):
        # loguru 会自动处理文件关闭，所以这里不需要特别的清理代码
        pass


from abc import ABC, abstractmethod
from typing import Optional

class IServerLog78(ABC):
    @property
    @abstractmethod
    def server_url(self) -> str:
        pass

    @server_url.setter
    @abstractmethod
    def server_url(self, value: str):
        pass

    @abstractmethod
    async def log_to_server(self, log_entry: dict) -> Optional[dict]:
        pass


import asyncio
import os
from enum import Enum
from typing import Optional, Dict, Any, Tuple, Union
from datetime import datetime
from .log_entry import LogEntry, BasicInfo, ErrorInfo
from .iserver_log78 import IServerLog78
from .iconsole_log78 import IConsoleLog78
from .ifile_log78 import IFileLog78
from .console_log78 import ConsoleLog78
from .file_log78 import FileLog78
from .file_log_detail import FileLogDetail
import traceback

class Environment(Enum):
    Production = 0
    Development = 1
    Testing = 2

class Log78:
    def __init__(self):
        self.debug_kind = set()
        self.level_file = 30
        self.level_console = 60
        self.level_api = 50
        self.server_logger: Optional[IServerLog78] = None
        self.console_logger: Optional[IConsoleLog78] = ConsoleLog78()
        self.file_logger: Optional[IFileLog78] = FileLog78()
        self.debug_file_logger: Optional[IFileLog78] = None
        self.debug_entry: Optional[LogEntry] = None
        self.current_environment = Environment.Production
        self.custom_properties: Dict[str, Any] = {}
        self._lock = asyncio.Lock()
        self._semaphore = asyncio.Semaphore(1)
        self.set_environment_from_env_var()

    @classmethod
    def instance(cls) -> 'Log78':
        if not hasattr(cls, "_instance"):
            cls._instance = Log78()
            cls._instance.setup(None, FileLog78(), ConsoleLog78())
        return cls._instance

    def set_environment_from_env_var(self):
        env_var = os.environ.get("LOG78_ENVIRONMENT")
        if env_var:
            try:
                self.set_environment(Environment[env_var])
            except KeyError:
                self.set_environment(Environment.Production)
        else:
            self.set_environment(Environment.Production)

    def set_environment(self, env: Environment):
        self.current_environment = env
        self.update_log_levels()
        self.setup_debug_file_logger()

    def update_log_levels(self):
        if self.current_environment == Environment.Production:
            self.level_console = 60  # ERROR
            self.level_file = 30     # INFO
            self.level_api = 30      # INFO
        elif self.current_environment == Environment.Development:
            self.level_console = 20  # DEBUG
            self.level_file = 20     # DEBUG
            self.level_api = 30      # INFO
        elif self.current_environment == Environment.Testing:
            self.level_console = 60  # ERROR
            self.level_file = 20     # DEBUG
            self.level_api = 30      # INFO

    def setup_debug_file_logger(self):
        if self.current_environment == Environment.Development:
            self.debug_file_logger = FileLogDetail()
        else:
            self.debug_file_logger = None

    def setup(self, server_logger: Optional[IServerLog78], file_logger: Optional[IFileLog78], console_logger: Optional[IConsoleLog78]):
        self.server_logger = server_logger
        self.file_logger = file_logger or self.file_logger
        self.console_logger = console_logger or self.console_logger

    def setup_detail_file(self):
        self.debug_file_logger = FileLogDetail()

    def clear_detail_log(self):
        if self.debug_file_logger:
            self.debug_file_logger.clear()

    def add_property(self, key: str, value: Any):
        self.custom_properties[key] = value

    async def process_log_internal(self, log_entry: Optional[LogEntry]):
        async with self._semaphore:
            if log_entry is None or log_entry.basic is None:
                await self.ERROR(LogEntry(basic=BasicInfo(message="Error: LogEntry or LogEntry.Basic is null")))
                return

            async with self._lock:
                for key, value in self.custom_properties.items():
                    log_entry.add_property(key, value)

            if self.debug_file_logger:
                self.debug_file_logger.log_to_file(log_entry)

            is_debug = self.is_debug_key(log_entry)

            if is_debug or log_entry.basic.log_level_number >= self.level_api:
                if self.server_logger:
                    try:
                        await self.server_logger.log_to_server(log_entry)
                    except Exception as ex:
                        print(f"Error in server logging: {ex}")

            if is_debug or log_entry.basic.log_level_number >= self.level_file:
                if self.file_logger:
                    self.file_logger.log_to_file(log_entry)

            if is_debug or log_entry.basic.log_level_number >= self.level_console:
                if self.console_logger:
                    self.console_logger.write_line(log_entry)  # 确保这行代码存在并且没有被注释掉

    def is_debug_key(self, log_entry: LogEntry) -> bool:
        if self.debug_entry and self.debug_entry.basic:
            return (self.debug_entry.basic.service_name and log_entry.basic.service_name and 
                    self.debug_entry.basic.service_name.lower() == log_entry.basic.service_name.lower()) or \
                   (self.debug_entry.basic.service_obj and log_entry.basic.service_obj and 
                    self.debug_entry.basic.service_obj.lower() == log_entry.basic.service_obj.lower()) or \
                   (self.debug_entry.basic.service_fun and log_entry.basic.service_fun and 
                    self.debug_entry.basic.service_fun.lower() == log_entry.basic.service_fun.lower()) or \
                   (self.debug_entry.basic.user_id and log_entry.basic.user_id and 
                    self.debug_entry.basic.user_id.lower() == log_entry.basic.user_id.lower()) or \
                   (self.debug_entry.basic.user_name and log_entry.basic.user_name and 
                    self.debug_entry.basic.user_name.lower() == log_entry.basic.user_name.lower())

        keys_to_check = [
            log_entry.basic.service_name,
            log_entry.basic.service_obj,
            log_entry.basic.service_fun,
            log_entry.basic.user_id,
            log_entry.basic.user_name
        ]

        return any(key and key.lower() in self.debug_kind for key in keys_to_check if key)

    async def DETAIL(self, summary_or_log_entry: Union[str, LogEntry], message: Any = None, level: int = 10):
        if isinstance(summary_or_log_entry, LogEntry):
            log_entry = summary_or_log_entry
            log_entry.basic.log_level = "DETAIL"
            log_entry.basic.log_level_number = level
        else:
            log_entry = LogEntry(basic=BasicInfo(
                summary=summary_or_log_entry,
                message=message,
                log_level="DETAIL",
                log_level_number=level
            ))
        await self.process_log_internal(log_entry)

    async def DEBUG(self, summary_or_log_entry: Union[str, LogEntry], message: Any = None, level: int = 20):
        if isinstance(summary_or_log_entry, LogEntry):
            log_entry = summary_or_log_entry
            log_entry.basic.log_level = "DEBUG"
            log_entry.basic.log_level_number = level
        else:
            log_entry = LogEntry(basic=BasicInfo(
                summary=summary_or_log_entry,
                message=message,
                log_level="DEBUG",
                log_level_number=level
            ))
        await self.process_log_internal(log_entry)

    async def INFO(self, summary_or_log_entry: Union[str, LogEntry], message: Any = None, level: int = 30):
        
        if isinstance(summary_or_log_entry, LogEntry):
            log_entry = summary_or_log_entry
            log_entry.basic.log_level = "INFO"
            log_entry.basic.log_level_number = level
        else:
            log_entry = LogEntry(basic=BasicInfo(
                summary=summary_or_log_entry,
                message=message,
                log_level="INFO",
                log_level_number=level
            ))
        
        await self.process_log_internal(log_entry)
        

    async def WARN(self, summary_or_log_entry: Union[str, LogEntry], message: Any = None, level: int = 50):
        if isinstance(summary_or_log_entry, LogEntry):
            log_entry = summary_or_log_entry
            log_entry.basic.log_level = "WARN"
            log_entry.basic.log_level_number = level
        else:
            log_entry = LogEntry(basic=BasicInfo(
                summary=summary_or_log_entry,
                message=message,
                log_level="WARN",
                log_level_number=level
            ))
        await self.process_log_internal(log_entry)

    async def ERROR(self, summary_or_log_entry: Union[str, LogEntry, Exception], message: Any = None, level: int = 60):
        if isinstance(summary_or_log_entry, LogEntry):
            log_entry = summary_or_log_entry
            log_entry.basic.log_level = "ERROR"
            log_entry.basic.log_level_number = level
        elif isinstance(summary_or_log_entry, Exception):
            log_entry = LogEntry(
                basic=BasicInfo(
                    summary=str(summary_or_log_entry),
                    message=message,
                    log_level="ERROR",
                    log_level_number=level
                ),
                error=ErrorInfo(
                    error_type=type(summary_or_log_entry).__name__,
                    error_message=str(summary_or_log_entry),
                    error_stack_trace=traceback.format_exc()
                )
            )
        else:
            log_entry = LogEntry(basic=BasicInfo(
                summary=summary_or_log_entry,
                message=message,
                log_level="ERROR",
                log_level_number=level
            ))
        await self.process_log_internal(log_entry)

    def setup_level(self, file_level: int, console_level: int, api_level: int):
        self.level_file = file_level
        self.level_console = console_level
        self.level_api = api_level

    def get_current_levels(self) -> Tuple[int, int, int]:
        return (self.level_file, self.level_console, self.level_api)

    def clone(self) -> 'Log78':
        cloned = Log78()
        cloned.server_logger = self.server_logger
        cloned.file_logger = self.file_logger
        cloned.console_logger = self.console_logger
        cloned.level_api = self.level_api
        cloned.level_console = self.level_console
        cloned.level_file = self.level_file
        cloned.current_environment = self.current_environment
        cloned.custom_properties = self.custom_properties.copy()  # 确保复制自定义属性
        return cloned

    async def add_debug_key(self, key: str):
        async with self._lock:
            self.debug_kind.add(key.lower())


import os
import json
from .log_entry import LogEntry

class FileLogDetail:
    def __init__(self, filename="detail.log", menu="logs"):
        self.file_path = os.path.join(menu, filename)
        os.makedirs(menu, exist_ok=True)      

    def log_to_file(self, log_entry: LogEntry):
        try:
            log_string = f"<AI_FOCUS_LOG>{log_entry.to_json()}</AI_FOCUS_LOG>\n"
            with open(self.file_path, 'a', encoding='utf-8') as f:  # 添加 encoding='utf-8'
                f.write(log_string)
        except Exception as ex:
            print(f"写入详细日志文件时出错: {ex}")

    def clear(self):
        with open(self.file_path, 'w', encoding='utf-8') as f:  # 添加 encoding='utf-8'
            f.write('')

    def close(self):
        # Python会自动关闭文件,所以这里不需要特别的操作
        pass


import json
from typing import Any, Dict, Optional
from datetime import datetime, timezone
import uuid
from dataclasses import dataclass, field

@dataclass
class BasicInfo:
    summary: str = ""
    message: Optional[str] = None
    log_level: str = "INFO"
    log_level_number: int = 30
    timestamp: datetime = field(default_factory=lambda: datetime.now(timezone.utc))
    service_name: Optional[str] = None
    service_obj: Optional[str] = None
    service_fun: Optional[str] = None
    user_id: Optional[str] = None
    user_name: Optional[str] = None

@dataclass
class EventInfo:
    event_id: str = field(default_factory=lambda: str(uuid.uuid4()))
    event_kind: Optional[str] = None
    event_category: Optional[str] = None
    event_action: Optional[str] = None
    event_outcome: Optional[str] = None
    event_duration: Optional[float] = None
    transaction_id: Optional[str] = None

@dataclass
class ErrorInfo:
    error_type: Optional[str] = None
    error_message: Optional[str] = None
    error_stack_trace: Optional[str] = None

@dataclass
class HttpInfo:
    http_request_method: Optional[str] = None
    http_request_body_content: Optional[str] = None
    http_response_status_code: Optional[int] = None
    url_original: Optional[str] = None

@dataclass
class TraceInfo:
    trace_id: Optional[str] = None
    span_id: Optional[str] = None

class LogEntry:
    def __init__(self, basic: BasicInfo = None, event: EventInfo = None, error: ErrorInfo = None, http: HttpInfo = None, trace: TraceInfo = None):
        self.basic = basic if basic is not None else BasicInfo()
        self.event = event if event is not None else EventInfo()
        self.error = error if error is not None else ErrorInfo()
        self.http = http if http is not None else HttpInfo()
        self.trace = trace if trace is not None else TraceInfo()
        self.additional_properties: Dict[str, Any] = {}

    def to_json(self) -> str:
        data = {}
        self._add_properties_to_dict(data, self)
        if 'additional_properties' in data and not data['additional_properties']:
            del data['additional_properties']
        return json.dumps(data, ensure_ascii=False, default=str)  # 添加 default=str

    def _add_properties_to_dict(self, data: Dict[str, Any], obj: Any):
        if isinstance(obj, dict):
            for key, value in obj.items():
                self._add_value_to_dict(data, key, value)
        elif hasattr(obj, '__dict__'):
            for key, value in obj.__dict__.items():
                if not key.startswith('_'):
                    self._add_value_to_dict(data, key, value)

    def _add_value_to_dict(self, data: Dict[str, Any], key: str, value: Any):
        if value is not None and str(value).strip():
            if isinstance(value, (BasicInfo, EventInfo, ErrorInfo, HttpInfo, TraceInfo)):
                self._add_properties_to_dict(data, value)
            elif isinstance(value, datetime):
                data[key.lower()] = value.isoformat()
            else:
                data[key.lower()] = value

    def add_property(self, key: str, value: Any):
        self.additional_properties[key] = value


import asyncio
import aiohttp
from datetime import datetime
from typing import Optional
from .log_entry import LogEntry, BasicInfo
from .log78 import Log78

class LogstashServerLog78:
    def __init__(self, server_url: str):
        self.server_url = server_url
        self._logger = Log78.instance()

    async def log_to_server(self, log_entry: LogEntry):
        try:
            return await asyncio.wait_for(self._log_to_server_internal(log_entry), timeout=30)
        except Exception as ex:
            error_log_entry = LogEntry()
            error_log_entry.basic = BasicInfo()
            error_log_entry.basic.summary = "Logstash Error"
            error_log_entry.basic.message = f"Unexpected error in LogToServer: {str(ex)}"
            await self._logger.ERROR(error_log_entry)

    async def _log_to_server_internal(self, log_entry: LogEntry):
        json_content = log_entry.to_json()
        async with aiohttp.ClientSession() as session:
            try:
                async with session.post(self.server_url, data=json_content, headers={'Content-Type': 'application/json'}, timeout=20) as response:
                    if response.status == 200:
                        success_log_entry = LogEntry()
                        success_log_entry.basic = BasicInfo()
                        success_log_entry.basic.summary = "Logstash Success"
                        success_log_entry.basic.message = "Logstash log sent successfully"
                        await self._logger.DEBUG(success_log_entry)
                    else:
                        error_message = f"Logstash server returned status code {response.status}"
                        error_log_entry = LogEntry()
                        error_log_entry.basic = BasicInfo()
                        error_log_entry.basic.summary = "Logstash Error"
                        error_log_entry.basic.message = error_message
                        await self._logger.ERROR(error_log_entry)
                    return response
            except asyncio.TimeoutError:
                timeout_log_entry = LogEntry()
                timeout_log_entry.basic = BasicInfo()
                timeout_log_entry.basic.summary = "Logstash Canceled"
                timeout_log_entry.basic.message = "HTTP request was canceled or timed out"
                await self._logger.ERROR(timeout_log_entry)
            except Exception as ex:
                error_message = f"Exception occurred while sending log to Logstash: {str(ex)}"
                exception_log_entry = LogEntry()
                exception_log_entry.basic = BasicInfo()
                exception_log_entry.basic.summary = "Logstash Exception"
                exception_log_entry.basic.message = error_message
                await self._logger.ERROR(exception_log_entry)
                raise

    async def __aenter__(self):
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        # 在这里可以添加清理代码,如果需要的话
        pass


import logging
import sys
from .log_entry import LogEntry

class ConsoleLog78:
    def __init__(self):
        self._logger = logging.getLogger(__name__)
        self._logger.setLevel(logging.INFO)
        handler = logging.StreamHandler(sys.stdout)  # 使用 sys.stdout
        formatter = logging.Formatter('%(message)s')
        handler.setFormatter(formatter)
        self._logger.addHandler(handler)

    def write_line(self, log_entry: LogEntry):
        self._logger.info(log_entry.to_json())

    def __del__(self):
        for handler in self._logger.handlers[:]:
            self._logger.removeHandler(handler)
            handler.close()


from .log78 import Log78, Environment
from .log_entry import LogEntry, BasicInfo, ErrorInfo, EventInfo, HttpInfo, TraceInfo
from .console_log78 import ConsoleLog78
from .file_log78 import FileLog78
from .file_log_detail import FileLogDetail
from .iserver_log78 import IServerLog78
from .iconsole_log78 import IConsoleLog78
from .ifile_log78 import IFileLog78
from .ilog78 import ILog78
from .logstash_server_log78 import LogstashServerLog78

__all__ = [
    'Log78',
    'Environment',
    'LogEntry',
    'BasicInfo',
    'ErrorInfo',
    'EventInfo',
    'HttpInfo',
    'TraceInfo',
    'ConsoleLog78',
    'FileLog78',
    'FileLogDetail',
    'IServerLog78',
    'IConsoleLog78',
    'IFileLog78',
    'ILog78',
    'LogstashServerLog78'
]


import asyncio
from log78 import Log78, LogEntry, BasicInfo, Environment
import sys
import io

sys.stdout = io.TextIOWrapper(sys.stdout.buffer, encoding='utf-8')
sys.stderr = io.TextIOWrapper(sys.stderr.buffer, encoding='utf-8')

async def demo_basic_logging():
    log = Log78.instance()
    log.set_environment(Environment.Development)

    await log.INFO("这是一条基本的信息日志")
    await log.WARN("这是一条警告日志", "警告摘要")
    await log.ERROR("这是一条错误日志", "错误摘要", 70)

async def demo_detailed_logging():
    log = Log78.instance()
    
    log_entry = LogEntry()
    log_entry.basic = BasicInfo(
        summary="用户登录",
        message="用户 John Doe 成功登录",
        service_name="AuthService",
        user_id="user123"
    )
    
    await log.INFO(log_entry)

async def test_file_logging():
    log = Log78.instance()
    log.set_environment(Environment.Development)

    await log.INFO("这是一条写入文件的中文日志测试")
    await log.WARN("警告：包含特殊字符 !@#$%^&*()_+")
    await log.ERROR("错误：包含 Unicode 字符 ♠♥♦♣")
    
    log_entry = LogEntry(basic=BasicInfo(
        summary="复杂日志条目",
        message="这是一个包含多种信息的日志条目",
        service_name="测试服务",
        user_id="user_123"
    ))
    await log.INFO(log_entry)

    print("请检查日志文件以确认中文和特殊字符是否正确显示")

async def main():
    print("Log78 演示程序")
    print("文件日志测试")
    await test_file_logging()

if __name__ == "__main__":
    asyncio.run(main())



from abc import ABC, abstractmethod
from typing import Optional, Union

from log78.iconsole_log78 import IConsoleLog78
from log78.ifile_log78 import IFileLog78
from log78.iserver_log78 import IServerLog78
from log78.log78 import Log78

class ILog78(ABC):
    @abstractmethod
    async def DETAIL(self, summary: str, message: Optional[Union[str, dict]] = None, level: int = 10):
        pass

    @abstractmethod
    async def DEBUG(self, summary: str, message: Optional[Union[str, dict]] = None, level: int = 20):
        pass

    @abstractmethod
    async def INFO(self, summary: str, message: Optional[Union[str, dict]] = None, level: int = 30):
        pass

    @abstractmethod
    async def WARN(self, summary: str, message: Optional[Union[str, dict]] = None, level: int = 50):
        pass

    @abstractmethod
    async def ERROR(self, summary: str, message: Optional[Union[str, dict]] = None, level: int = 60):
        pass

    @abstractmethod
    async def ERROR(self, error: Exception, summary: Optional[str] = None, level: int = 60):
        pass

    @abstractmethod
    async def ERROR(self, log_entry: dict, level: int = 60):
        pass

    @abstractmethod
    async def DEBUG(self, log_entry: dict, level: int = 20):
        pass

    @abstractmethod
    async def INFO(self, log_entry: dict, level: int = 30):
        pass

    @abstractmethod
    async def WARN(self, log_entry: dict, level: int = 50):
        pass

    @abstractmethod
    def setup(self, server_logger: Optional['IServerLog78'], file_logger: Optional['IFileLog78'], console_logger: Optional['IConsoleLog78']):
        pass

    @abstractmethod
    def set_environment(self, env: 'Log78.Environment'):
        pass

    @abstractmethod
    def setup_level(self, file_level: int, console_level: int, api_level: int):
        pass

    @abstractmethod
    def add_debug_key(self, key: str):
        pass

    @abstractmethod
    def clear_detail_log(self):
        pass

    @abstractmethod
    def setup_detail_file(self):
        pass